package org.bitbucket.davidm24.mongodb.aggregate;

import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;

/**
 * Utility for creating {@code $group} pipeline operations.
 * 
 * <p>
 * In general usage, you start a new group pipeline with {@link #start}. You
 * then call {@link #put} to specify a target field, followed by a group
 * aggregation function to specify how to populate that field. Repeat this for
 * each target field, and then [pass it to {@link AggregateBuilder#group} add it
 * to the pipeline.
 * </p>
 * 
 * <p>For example, consider the following JavaScript example, converted to Java:
 * <pre>JavaScript:
 * { $group : {
 *     _id : "$author",
 *     docsPerAuthor : { $sum : 1 },
 *     viewsPerAuthor : { $sum : "$pageViews" }
 * }}</pre>
 * <pre>Java:
 * GroupBuilder.start()
 *    .put("_id").to("$author")
 *    .put("docsPerAuthor").sum(1)
 *    .put("viewsPerAuthor").sum("$pageViews")</pre>
 * </p>
 * 
 * @author David M. Carr
 * @see <a href="http://docs.mongodb.org/manual/reference/aggregation/">Aggregation Framework Reference</a>
 * @since 0.1.0
 */
public class GroupBuilder {
    /**
     * A violation of the contract of GroupBuilder.
     * 
     * @author David M. Carr
     * @since 0.1.0
     */
    @SuppressWarnings("serial")
    public static class GroupBuilderException extends RuntimeException {
        GroupBuilderException(String message) {
            super(message);
        }
    }

    private static class NullObject {
    }

    /**
     * Returns a new GroupBuilder.  Initially, no target field is set.
     * 
     * @see #push(String)
     * @return a new GroupBuilder
     * @since 0.1.0
     */
    public static GroupBuilder start() {
        return new GroupBuilder();
    }

    /**
     * Returns a new GroupBuilder with an initial target field.  This is
     * equivalent to {@link #start} followed by {@link #push}.
     * 
     * @param targetField the desired initial target field
     * @return a new GroupBuilder
     * @since 0.1.0
     */
    public static GroupBuilder start(String targetField) {
        return start().put(targetField);
    }

    private DBObject _body;
    private String _targetField;

    /**
     * Instantiates a new GroupBuilder.
     * 
     * @since 0.1.0
     */
    public GroupBuilder() {
        _body = new BasicDBObject();
    }

    private void addOperand(String op, Object value) {
        if (_targetField == null) {
            throw new GroupBuilderException("No target field for operand");
        }
        if (op == null) {
            _body.put(_targetField, value);
        } else {
            _body.put(_targetField, new BasicDBObject(op, value));
        }
    }

    /**
     * Applies the {@code $addToSet} group aggregation function to the specified
     * source field.
     * 
     * <p>This function returns an array of all the values found in the selected
     * field among the documents in that group.  Every unique value only appears
     * once in the result set.  There is no ordering guarantee for the output
     * documents.</p> 
     * 
     * @param sourceField the source field upon which to operate
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder addToSet(String sourceField) {
        addOperand(GroupOperators.ADD_TO_SET, sourceField);
        return this;
    }

    /**
     * Applies the {@code $avg} group aggregation function to the specified
     * source field.
     * 
     * <p>This function returns the average of all the values of the field in
     * all documents selected by this group.</p>
     * 
     * @param sourceField the source field upon which to operate
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder avg(String sourceField) {
        addOperand(GroupOperators.AVG, sourceField);
        return this;
    }

    /**
     * Applies the {@code $first} group aggregation function to the specified
     * source field.
     * 
     * <p>This function returns the first value it encounters for its group.</p>
     * 
     * @param sourceField the source field upon which to operate
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder first(String sourceField) {
        addOperand(GroupOperators.FIRST, sourceField);
        return this;
    }

    /**
     * Creates a {@link DBObject} to be passed to a {@code $group} operator.
     * 
     * @return a {@link DBObject} instance
     * @throws GroupBuilderException if a key does not have a matching operand
     * @since 0.1.0
     */
    public DBObject get() {
        for (String key : _body.keySet()) {
            if (_body.get(key) instanceof NullObject) {
                throw new GroupBuilderException("No operand for key: " + key);
            }
        }
        return _body;
    }

    /**
     * Applies the {@code $last} group aggregation function to the specified
     * source field.
     * 
     * <p>This function returns the last value it encounters for its group.</p>
     * 
     * @param sourceField the source field upon which to operate
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder last(String sourceField) {
        addOperand(GroupOperators.LAST, sourceField);
        return this;
    }

    /**
     * Applies the {@code $max} group aggregation function to the specified
     * source field.
     * 
     * <p>This function returns the highest value among all values of the field
     * in all documents selected by this group.</p>
     * 
     * @param sourceField the source field upon which to operate
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder max(String sourceField) {
        addOperand(GroupOperators.MAX, sourceField);
        return this;
    }

    /**
     * Applies the {@code $min} group aggregation function to the specified
     * source field.
     * 
     * <p>This function returns the lowest value among all values of the field
     * in all documents selected by this group.</p>
     * 
     * @param sourceField the source field upon which to operate
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder min(String sourceField) {
        addOperand(GroupOperators.MIN, sourceField);
        return this;
    }

    /**
     * Applies the {@code $push} group aggregation function to the specified
     * source field.
     * 
     * <p>This function returns an array of all the values found in the selected
     * field among the documents in that group. A value may appear more than
     * once in the result set if more than one field in the grouped documents
     * has that value.</p>
     * 
     * @param sourceField the source field upon which to operate
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder push(String sourceField) {
        addOperand(GroupOperators.PUSH, sourceField);
        return this;
    }

    /**
     * Sets a new target field to which subsequent group aggregation functions
     * will apply.
     * 
     * @param targetField  the desired target field
     * @return the GroupBuilder instance for call chaining
     * @since 0.1.0
     */
    public GroupBuilder put(String targetField) {
        _targetField = targetField;
        if (_body.get(targetField) == null) {
            _body.put(_targetField, new NullObject());
        }
        return this;
    }

    /**
     * Applies the {@code $sum} group aggregation function to the specified
     * constant value.
     * 
     * <p>This function will increment this field by the specified value for
     * every document in the grouping. Typically specify a value of 1 in order
     * to count members of the group.</p>
     * 
     * @param num the value by which to increment the target field
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder sum(int num) {
        addOperand(GroupOperators.SUM, num);
        return this;
    }

    /**
     * Applies the {@code $sum} group aggregation function to the specified
     * source field.
     * 
     * <p>This function returns the sum of all the values for a specified field
     * in the grouped documents.</p>
     * 
     * @param sourceField the source field upon which to operate
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder sum(String sourceField) {
        addOperand(GroupOperators.SUM, sourceField);
        return this;
    }

    /**
     * Sets the target field to the specified value.  The value is usually
     * either a document (as a {@link DBObject} or {@link java.util.Map}) or a
     * field reference.  In particular, this is useful for setting the
     * {@code _id} field.
     * 
     * @param value the value to set for the target field
     * @return the GroupBuilder instance for call chaining
     * @throws GroupBuilderException if no target field was specified using
     *          {@link #push}
     * @since 0.1.0
     */
    public GroupBuilder to(Object value) {
        addOperand(null, value);
        return this;
    }
}
