/**
 * Extends the MongoDB Java Driver to provide an easier syntax for working
 * with the
 * <a href="http://docs.mongodb.org/manual/applications/aggregation/">Aggregation Framework</a>.
 * 
 * <p>For example, the
 * <a href="http://docs.mongodb.org/manual/tutorial/aggregation-examples/">Aggregation Framework Examples</a>
 * include the following "Largest and Smallest Cities by State" JavaScript
 * example:
<pre>db.zipcodes.aggregate(
        { $group: {
                _id: { state: "$state", city: "$city" },
                pop: { $sum: "$pop" }
        } },
        { $sort: { pop: 1 } },
        { $group: {
                _id: "$_id.state",
                biggestCity: { $last: "$_id.city" },
                biggestPop: { $last: "$pop" },
                smallestCity: { $first: "$_id.city" },
                smallestPop: { $first: "$pop" }
        } },
        { $project: {
                _id: 0,
                state: "$_id",
                biggestCity:  { name: "$biggestCity",  pop: "$biggestPop" },
                smallestCity: { name: "$smallestCity", pop: "$smallestPop" }
        } } )</pre>
 * Using this library this could be written in Java as:
<pre>AggregateBuilder.aggregate(db.getCollection("zipcodes"))
        .group(GroupBuilder.start()
                 .put("_id").to(DBO.of("state", "$state", "city", "$city"))
                 .put("pop").sum("$pop")
        )
        .sort(DBO.of("pop", 1))
        .group(GroupBuilder.start()
                .put("_id").to("$_id.state")
                .put("biggestCity").last("$_id.city")
                .put("biggestPop").last("$pop")
                .put("smallestCity").first("$_id.city")
                .put("smallestPop").first("$pop")
        )
        .project(DBO.of(
                "_id", 0,
                "state", "$_id",
                "biggestCity",
                DBO.of("name", "$biggestCity", "pop", "$biggestPop"),
                "smallestCity",
                DBO.of("name", "$smallestCity", "pop", "$smallestPop")
        ))
        .run();</pre>
 * As may be clear from this example, use of the aggregation framework goes a
 * lot more smoothly with easy ways to create maps.  In Groovy, this isn't an
 * issue, as a concise map syntax is built into the language.  In Java, I've provided
 * {@link org.bitbucket.davidm24.mongodb.aggregate.DBO#of} to bridge this gap.
 * Another alternative is to use {@link com.mongodb.BasicDBObject} and chain the append method.</p>
 * 
 * @author David M. Carr
 * @see <a href="http://docs.mongodb.org/manual/reference/aggregation/">Aggregation Framework Reference</a>
 * @since 0.1.0
 */
package org.bitbucket.davidm24.mongodb.aggregate;
